@@ -16,8 +16,9 @@ from rich.table import Table
 from structlog.stdlib import LoggerFactory
 
 from goodbase.build_checker import BuildChecks
-from goodbase.services.config_service import ConfigurationService
+from goodbase.services.config_service import ConfigurationService, CriteriaConfiguration
 from goodbase.services.evg_service import EvergreenService
+from goodbase.services.file_service import FileService
 from goodbase.services.git_service import GitAction, GitService
 
 LOGGER = structlog.get_logger(__name__)
@@ -39,15 +40,46 @@ class GoodBaseOptions(NamedTuple):
     Options for execution.
 
     * max_lookback: Number of commits to scan before giving up.
+    * commit_limit: Oldest commit to look at before giving up.
     * operation: Type of git operation to perform.
     * override_criteria: Override conflicting save criteria.
     * timeouts_secs: Number of seconds to scan before timing out.
+    * branch_name: Name of branch to create on checkout.
     """
 
     max_lookback: int
+    commit_limit: Optional[str]
     operation: GitAction
     override_criteria: bool
     timeout_secs: Optional[int] = None
+    branch_name: Optional[str] = None
+
+    def lookback_limit_hit(self, index: int, revision: str, elapsed_seconds: float) -> bool:
+        """
+        Determine if the limits of looking back have been hit.
+
+        :param index: Index of version being checked.
+        :param revision: git revision being checked.
+        :param elapsed_seconds: Number of seconds that have passed since operation started.
+        :return: True if we have hit the limit of version of check.
+        """
+        if index > self.max_lookback:
+            LOGGER.debug("Max lookback hit", max_lookback=self.max_lookback, commit_idx=index)
+            return True
+
+        if self.commit_limit and revision.startswith(self.commit_limit):
+            LOGGER.debug("Commit limit hit", commit_limit=self.commit_limit)
+            return True
+
+        if self.timeout_secs and elapsed_seconds > self.timeout_secs:
+            LOGGER.debug(
+                "Timeout hit",
+                timeout_secs=self.timeout_secs,
+                elapsed_seconds=elapsed_seconds,
+            )
+            return True
+
+        return False
 
 
 class RevisionInformation(NamedTuple):
@@ -74,6 +106,7 @@ class GoodBaseOrchestrator:
         evg_service: EvergreenService,
         git_service: GitService,
         config_service: ConfigurationService,
+        file_service: FileService,
         options: GoodBaseOptions,
         console: Console,
     ) -> None:
@@ -84,6 +117,7 @@ class GoodBaseOrchestrator:
         :param evg_service:  Evergreen Service.
         :param git_service: Git Service.
         :param config_service: Configuration service.
+        :param file_service: File service.
         :param options: Options for execution.
         :param console: Rich console to print to.
         """
@@ -91,6 +125,7 @@ class GoodBaseOrchestrator:
         self.evg_service = evg_service
         self.git_service = git_service
         self.config_service = config_service
+        self.file_service = file_service
         self.options = options
         self.console = console
 
@@ -109,10 +144,9 @@ class GoodBaseOrchestrator:
             label=f"Searching {evg_project} revisions",
         ) as bar:
             for idx, evg_version in enumerate(bar):
-                if idx > self.options.max_lookback:
-                    LOGGER.debug(
-                        "Max lookback hit", max_lookback=self.options.max_lookback, commit_idx=idx
-                    )
+                current_time = perf_counter()
+                elapsed_time = current_time - start_time
+                if self.options.lookback_limit_hit(idx, evg_version.revision, elapsed_time):
                     return None
 
                 LOGGER.debug("Checking version", commit=evg_version.revision)
@@ -120,16 +154,6 @@ class GoodBaseOrchestrator:
                 if self.evg_service.check_version(evg_version, build_checks):
                     return evg_version.revision
 
-                current_time = perf_counter()
-                elapsed_time = current_time - start_time
-                if self.options.timeout_secs and elapsed_time > self.options.timeout_secs:
-                    LOGGER.debug(
-                        "Timeout hit",
-                        timeout_secs=self.options.timeout_secs,
-                        elapsed_time=elapsed_time,
-                    )
-                    return None
-
         return None
 
     def attempt_git_operation(
@@ -144,7 +168,9 @@ class GoodBaseOrchestrator:
         :return: Error message if an error was encountered.
         """
         try:
-            self.git_service.perform_action(operation, revision, directory)
+            self.git_service.perform_action(
+                operation, revision, directory, self.options.branch_name
+            )
         except ProcessExecutionError:
             LOGGER.warning("Error encountered during git operation", exc_info=True)
             return f"Encountered error performing '{operation}' on '{revision}'"
@@ -223,6 +249,33 @@ class GoodBaseOrchestrator:
             raise ValueError("Not criteria found")
         return criteria.rules
 
+    def export_criteria(self, rules: List[str], destination: Path) -> None:
+        """
+        Export the given rules to the destination file.
+
+        :param rules: Names of rules to export.
+        :param destination: Path of file to export to.
+        """
+        rules_set = set(rules)
+        configuration = self.config_service.get_config()
+        rules_to_export = [rule for rule in configuration.saved_criteria if rule.name in rules_set]
+        export_config = CriteriaConfiguration(saved_criteria=rules_to_export)
+        self.file_service.write_yaml_file(destination, export_config.dict(exclude_none=True))
+
+    def import_criteria(self, import_file: Path) -> None:
+        """
+        Import rules from the given file.
+
+        :param import_file: File containing rules to import.
+        """
+        configuration = self.config_service.get_config()
+        import_file_contents = self.file_service.read_yaml_file(import_file)
+        import_criteria = CriteriaConfiguration(**import_file_contents)
+        for rule in import_criteria.saved_criteria:
+            for criteria in rule.rules:
+                configuration.add_criteria(rule.name, criteria, self.options.override_criteria)
+        self.config_service.save_config(configuration)
+
     def display_criteria(self) -> None:
         """Display saved criteria."""
         configuration = self.config_service.get_config()
@@ -305,12 +358,18 @@ def configure_logging(verbose: bool) -> None:
     help="Number of commits to check before giving up",
 )
 @click.option("--timeout-secs", type=int, help="Number of seconds to search for before giving up.")
+@click.option(
+    "--commit-limit",
+    type=str,
+    help="Oldest commit to check before giving up.",
+)
 @click.option(
     "--git-operation",
     type=click.Choice([a.value for a in GitAction]),
     default=GitAction.CHECKOUT,
     help="Git operations to perform with found commit [default=checkout].",
 )
+@click.option("-b", "--branch", help="Name of branch to create on checkout.")
 @click.option(
     "--save-criteria",
     type=str,
@@ -324,6 +383,13 @@ def configure_logging(verbose: bool) -> None:
     default=False,
     help="Override saved conflicting save criteria rules.",
 )
+@click.option(
+    "--export-criteria", multiple=True, help="Specify saved criteria to export to a file."
+)
+@click.option("--export-file", type=click.Path(), help="File to write exported rules to.")
+@click.option(
+    "--import-criteria", type=click.Path(exists=True), help="Import previously exported criteria."
+)
 @click.option("--verbose", is_flag=True, default=False, help="Enable debug logging.")
 def main(
     passing_task: List[str],
@@ -334,11 +400,16 @@ def main(
     evg_project: str,
     build_variant: List[str],
     commit_lookback: int,
+    commit_limit: Optional[str],
     timeout_secs: Optional[int],
     git_operation: GitAction,
+    branch: Optional[str],
     save_criteria: Optional[str],
     use_criteria: Optional[str],
     list_criteria: bool,
+    export_criteria: List[str],
+    export_file: str,
+    import_criteria: Optional[str],
     override: bool,
     verbose: bool,
 ) -> None:
@@ -394,9 +465,11 @@ def main(
 
     options = GoodBaseOptions(
         max_lookback=commit_lookback,
+        commit_limit=commit_limit,
         operation=git_operation,
         override_criteria=override,
         timeout_secs=timeout_secs,
+        branch_name=branch,
     )
 
     build_variant_checks = [".*-required$"]
@@ -439,6 +512,23 @@ def main(
             click.echo(click.style(str(err), fg="red"))
             sys.exit(1)
 
+    elif export_criteria:
+        if not export_file:
+            click.echo(
+                click.style("Export file needs to be specified with `--export-file`", fg="red")
+            )
+            sys.exit(1)
+
+        orchestrator.export_criteria(export_criteria, Path(export_file))
+
+    elif import_criteria:
+        try:
+            orchestrator.import_criteria(Path(import_criteria))
+        except ValueError as err:
+            click.echo(click.style(f"Could not import from: {import_criteria}", fg="red"))
+            click.echo(click.style(str(err), fg="red"))
+            sys.exit(1)
+
     else:
         criteria = [build_checks]
         if use_criteria:
